##Library Header
# $Id: $
# Copyright 2003-2006 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_igmpvx_api.tcl
#
# Purpose:
#     A script development library containing IGMPvX APIs for test automation
#     with the Ixia chassis.
#
# Author:
#    Lavinia Raicea
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_igmp_config
#    - emulation_igmp_querier_config
#    - emulation_igmp_control
#    - emulation_igmp_group_config
#    - emulation_igmp_info
#
# Requirements:
#     ixiaapiutils.tcl , a library containing TCL utilities
#     parseddashedargs.tcl , a library containing the proceDescr and
#     parsedashedargds.tcl
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################


proc ::ixia::emulation_igmp_config { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api

    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_igmp_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    variable igmp_host_handles_array
    variable igmp_group_handles_array
    variable igmp_attributes_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode CHOICES create delete modify enable disable enable_all disable_all
    }
    set opt_args {
        -atm_encapsulation              CHOICES VccMuxIPV4Routed
                                        CHOICES VccMuxIPV6Routed
                                        CHOICES VccMuxBridgedEthernetFCS
                                        CHOICES VccMuxBridgedEthernetNoFCS
                                        CHOICES LLCRoutedCLIP
                                        CHOICES LLCBridgedEthernetFCS
                                        CHOICES LLCBridgedEthernetNoFCS
        -count                          NUMERIC
                                        DEFAULT 1
        -enable_packing                 CHOICES 0 1
                                        DEFAULT 0
        -filter_mode                    CHOICES include exclude
        -general_query                  CHOICES 0 1
        -group_query                    CHOICES 0 1
        -handle
        -igmp_version                   CHOICES v1 v2 v3
        -interface_handle
        -intf_ip_addr                   IP
        -intf_ip_addr_step              IP
                                        DEFAULT 0.0.0.1
        -intf_prefix_len                RANGE   1-32
                                        DEFAULT 24
        -ip_router_alert                CHOICES 0 1
        -mac_address_init               MAC
        -mac_address_step               MAC 
                                        DEFAULT 0000.0000.0001
        -max_groups_per_pkts            RANGE   0-1500
        -max_response_control           CHOICES 0 1
        -max_response_time              NUMERIC
        -max_sources_per_group          RANGE   0-1500
        -msg_count_per_interval         NUMERIC
                                        DEFAULT 0
        -msg_interval                   NUMERIC
                                        DEFAULT 0
        -neighbor_intf_ip_addr          IP
        -neighbor_intf_ip_addr_step     IP
                                        DEFAULT 0.0.0.0
        -no_write                       FLAG
        -override_existence_check       CHOICES 0 1
                                        DEFAULT 0
        -override_tracking              CHOICES 0 1
                                        DEFAULT 0
        -port_handle                    REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -reset                          FLAG
        -suppress_report                CHOICES 0 1
        -unsolicited_report_interval    NUMERIC
        -vci                            RANGE   0-65535
                                        DEFAULT 10
        -vci_step                       RANGE   0-65535
                                        DEFAULT 1
        -vlan                           CHOICES 0 1
        -vlan_id                        RANGE   0-4095
        -vlan_id_mode                   CHOICES fixed increment
                                        DEFAULT increment
        -vlan_id_step                   RANGE   0-4096
                                        DEFAULT 1
        -vlan_user_priority             RANGE   0-7
                                        DEFAULT 0
        -vpi                            RANGE   0-255
                                        DEFAULT 1
        -vpi_step                       RANGE   0-255
                                        DEFAULT 1
    }
    
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_igmp_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}

    return $returnList
}


proc ::ixia::emulation_igmp_querier_config { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api

    set procName [lindex [info level [info level]] 0]
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_igmp_querier_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode CHOICES create delete modify enable disable
    }
    set opt_args {
        -atm_encapsulation                  CHOICES VccMuxIPV4Routed
                                            CHOICES VccMuxIPV6Routed
                                            CHOICES VccMuxBridgedEthernetFCS
                                            CHOICES VccMuxBridgedEthernetNoFCS
                                            CHOICES LLCRoutedCLIP
                                            CHOICES LLCBridgedEthernetFCS
                                            CHOICES LLCBridgedEthernetNoFCS
        -count                              NUMERIC
                                            DEFAULT 1
        -discard_learned_info               CHOICES 0 1
        -general_query_response_interval    RANGE 1-3174400
        -handle
        -igmp_version                       CHOICES v1 v2 v3
        -interface_handle
        -intf_ip_addr                       IP
        -intf_ip_addr_step                  IP
                                            DEFAULT 0.0.0.1
        -intf_prefix_len                    RANGE   1-32
                                            DEFAULT 24
        -ip_router_alert                    CHOICES 0 1
        -mac_address_init                   MAC
        -mac_address_step                   MAC 
                                            DEFAULT 0000.0000.0001
        -msg_count_per_interval             NUMERIC
        -msg_interval                       NUMERIC
        -neighbor_intf_ip_addr              IP
        -neighbor_intf_ip_addr_step         IP
                                            DEFAULT 0.0.0.0
        -no_write                           FLAG
        -override_existence_check           CHOICES 0 1
                                            DEFAULT 0
        -override_tracking                  CHOICES 0 1
                                            DEFAULT 0
        -port_handle                        REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -reset                              FLAG
        -robustness_variable                RANGE   1-7
        -specific_query_response_interval   RANGE   1-3174400
        -specific_query_transmission_count  RANGE   1-255
        -startup_query_count                RANGE   1-255
        -support_election                   CHOICES 0 1
        -support_older_version_host         CHOICES 0 1
        -support_older_version_querier      CHOICES 0 1
        -vci                                RANGE   0-65535
                                            DEFAULT 10
        -vci_step                           RANGE   0-65535
                                            DEFAULT 1
        -vlan                               CHOICES 0 1
        -vlan_id                            RANGE   0-4095
        -vlan_id_mode                       CHOICES fixed increment
                                            DEFAULT increment
        -vlan_id_step                       RANGE   0-4096
                                            DEFAULT 1
        -vlan_user_priority                 RANGE   0-7
                                            DEFAULT 0
        -vpi                                RANGE   0-255
                                            DEFAULT 1
        -vpi_step                           RANGE   0-255
                                            DEFAULT 1
        -query_interval                     RANGE   1-31744
    }
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
    
        set returnList [::ixia::ixnetwork_igmp_querier_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: [keylget returnList log]"
        }
        return $returnList
    } else {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: emulation_igmp_querier_config\
                is only supported for IxNetwork tcl API. "
        return $returnList
    }
}


proc ::ixia::emulation_igmp_control { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api
    
    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_igmp_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode           CHOICES stop start restart join leave
    }
    set opt_args {
        -group_member_handle
        -handle
        -port_handle    REGEXP ^[0-9]+/[0-9]+/[0-9]+$
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_igmp_control $args $man_args \
                $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_igmp_group_config { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api
    variable igmp_port
    
    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_igmp_group_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode            CHOICES create delete modify clear_all enable disable
    }
    set opt_args {
        -g_enable_packing        CHOICES  0 1
        -g_filter_mode           CHOICES  include exclude
        -g_max_groups_per_pkts   NUMERIC
        -g_max_sources_per_group RANGE    0-1500
        -group_pool_handle
        -handle
        -no_write                FLAG
        -reset                   FLAG
        -session_handle
        -source_pool_handle
    }
    
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_igmp_group_config $args $man_args \
                $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_igmp_info { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api
    
    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_igmp_info $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    # igmp_over_ppp and igmp are kept for backward compatibility they were
    #     deprecated and they refering to host option in the previous release
    #     (the same behaiviour will apply, but we removed them from docs) 
    #     Also a warning that the parameter is deprecated will be shown
    #     in ::ixia::ixnetwork_igmp_info parse_dashed_args
    set opt_args {
        -handle         REGEXP  ^::ixNet::OBJ-/vport:[0-9]+/protocols/igmp/querier:[0-9]+$
        -mode           CHOICES aggregate learned_info clear_stats
                        DEFAULT aggregate
        -port_handle    REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -timeout        NUMERIC
        -type           CHOICES igmp_over_ppp igmp host querier both
                        DEFAULT host
    }
    
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_igmp_info $args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
        set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
        return $returnList
    }
}
